rm(list = ls())

path  <-  getwd()
setwd(path)

choicedata <- read.csv('choice.data.csv')
biddata <- read.csv('bid.data.csv')
bio.data <- read.csv('bio.csv')



##load the package
library(lme4)
library(lmerTest)
library(rcompanion)
library(doBy)
library(effsize)


library(reshape2)


library(effects)


########################################################
##data preparation of bidding task
a1 <- subset(biddata, biddata$option == 1)
a2 <- subset(biddata, biddata$option == 2)
a3 <- merge(a1, a2, by = c("participant", "item"), all = F)

bid.merge <- data.frame(participant = a3$participant, item = a3$item, 
                       SSTotalFixationCount = a3$TotalFixationCount.x, SSTotalDwellTime = a3$TotalDwellTime.x, 
                       SSMeanFixationDuration = a3$MeanFixationDuration.x, SSOutcomeCount = a3$OutcomeCount.x,
                       SSDelayCount = a3$DelayCount.x, SSOutcomeDwell = a3$OutcomeDwell.x, SSDelayDwell = a3$DelayDwell.x,
                       SSRT = a3$RT.x, SSvaluation = a3$valuation.x, 
                       SStrialindex = a3$trialindex.x, SSoutcome = a3$outcome.x, SSdelay = a3$delay.x,
                       SSDiscountRate = a3$DiscountRate.x, 
                       LLTotalFixationCount = a3$TotalFixationCount.y, LLTotalDwellTime = a3$TotalDwellTime.y, 
                       LLMeanFixationDuration = a3$MeanFixationDuration.y, LLOutcomeCount = a3$OutcomeCount.y,
                       LLDelayCount = a3$DelayCount.y, LLOutcomeDwell = a3$OutcomeDwell.y, LLDelayDwell = a3$DelayDwell.y,
                       LLRT = a3$RT.y, LLvaluation = a3$valuation.y, 
                       LLtrialindex = a3$trialindex.y, LLoutcome = a3$outcome.y, LLdelay = a3$delay.y, 
                       LLDiscountRate = a3$DiscountRate.y)
bid.merge$SSBigerValuation <- ifelse(bid.merge$SSvaluation > bid.merge$LLvaluation, 1, 0)
bid.merge$SSBigerValuation[bid.merge$SSvaluation == bid.merge$LLvaluation] <- 2
rm(a1, a2, a3)



########################################################
##data preparation of choice task
a1 <- subset(choicedata, choicedata$SSleft == 1)
a2 <- subset(choicedata, choicedata$SSleft == 0)
a3 <- merge(a1, a2, by = c("participant", "item"), all = F)

choice.merge <- data.frame(participant = a3$participant, item = a3$item, 
                          SSTotalFixationCount = a3$TotalFixationCount.x, SSTotalDwellTime = a3$TotalDwellTime.x, 
                          SSMeanFixationDuration = a3$MeanFixationDuration.x, SSOutcomeCount = a3$OutcomeCount.x,
                          SSDelayCount = a3$DelayCount.x, SSOutcomeDwell = a3$OutcomeDwell.x, SSDelayDwell = a3$DelayDwell.x,
                          SSRT = a3$RT.x, SStrialindex = a3$trialindex.x, 
                          SSOutcomeDwell = a3$OutcomeDwell.x, SSDelayDwell = a3$DelayDwell.x,
                          LLTotalFixationCount = a3$TotalFixationCount.y, LLTotalDwellTime = a3$TotalDwellTime.y, 
                          LLMeanFixationDuration = a3$MeanFixationDuration.y, LLOutcomeCount = a3$OutcomeCount.y,
                          LLDelayCount = a3$DelayCount.y, LLOutcomeDwell = a3$OutcomeDwell.y, LLDelayDwell = a3$DelayDwell.y,
                          LLRT = a3$RT.y, LLtrialindex = a3$trialindex.y, 
                          LLOutcomeDwell = a3$OutcomeDwell.y, LLDelayDwell = a3$DelayDwell.y,
                          SSoutcome = a3$SSoutcome.x, SSdelay = a3$SSdelay.x, LLoutcome = a3$LLoutcome.x, LLdelay = a3$LLdelay.x,
                          SSchooseSS = a3$chooseSS.x, LLchooseSS = a3$chooseSS.y)
choice.merge$chooseSS <- 2
choice.merge$chooseSS[choice.merge$SSchooseSS == 1 & choice.merge$LLchooseSS == 1] <- 1
choice.merge$chooseSS[choice.merge$SSchooseSS == 0 & choice.merge$LLchooseSS == 0] <- 0

rm(a1, a2, a3)


########################################################
##merge the data

#The bidding task: a1
a1 <- data.frame(participant = bid.merge$participant, item = bid.merge$item, 
                 SSOutcomeDwell = bid.merge$SSOutcomeDwell, SSDelayDwell = bid.merge$SSDelayDwell, 
                 LLOutcomeDwell = bid.merge$LLOutcomeDwell, LLDelayDwell = bid.merge$LLDelayDwell,
                 SSRT = bid.merge$SSRT, LLRT = bid.merge$LLRT,
                 SSBigerValuation = bid.merge$SSBigerValuation,
                 SSoutcome = bid.merge$SSoutcome, SSdelay = bid.merge$SSdelay, 
                 LLoutcome = bid.merge$LLoutcome, LLdelay = bid.merge$LLdelay)
a1[is.na(a1)] <- 0
a1$SSDwell <- a1$SSOutcomeDwell + a1$SSDelayDwell
a1$LLDwell <- a1$LLOutcomeDwell + a1$LLDelayDwell
a1$OutcomeDwell <- a1$SSOutcomeDwell + a1$LLOutcomeDwell
a1$DelayDwell <- a1$SSDelayDwell + a1$LLDelayDwell

#The choice task: a2
a2 <- data.frame(participant = choice.merge$participant, item = choice.merge$item, 
                 SSOutcomeDwell = choice.merge$SSOutcomeDwell, SSDelayDwell = choice.merge$SSDelayDwell, 
                 LLOutcomeDwell = choice.merge$LLOutcomeDwell, LLDelayDwell = choice.merge$LLDelayDwell,
                 SSRT = choice.merge$SSRT, LLRT = choice.merge$LLRT,
                 chooseSS = choice.merge$chooseSS, 
                 SSoutcome = choice.merge$SSoutcome, SSdelay = choice.merge$SSdelay, 
                 LLoutcome = choice.merge$LLoutcome, LLdelay = choice.merge$LLdelay)
a2[is.na(a2)] <- 0
a2$SSDwell <- a2$SSOutcomeDwell + a2$SSDelayDwell
a2$LLDwell <- a2$LLOutcomeDwell + a2$LLDelayDwell
a2$OutcomeDwell <- a2$SSOutcomeDwell + a2$LLOutcomeDwell
a2$DelayDwell <- a2$SSDelayDwell + a2$LLDelayDwell

merge.data <- merge(a1, a2, by = c("participant", "item"), all = F)
merge.data$bidOGP <- (merge.data$SSOutcomeDwell.x + merge.data$LLOutcomeDwell.x) / (merge.data$SSOutcomeDwell.x + merge.data$LLOutcomeDwell.x + merge.data$SSDelayDwell.x + merge.data$LLDelayDwell.x)
merge.data$choiceOGP <- (merge.data$SSOutcomeDwell.y + merge.data$LLOutcomeDwell.y) / (merge.data$SSOutcomeDwell.y + merge.data$LLOutcomeDwell.y + merge.data$SSDelayDwell.y + merge.data$LLDelayDwell.y)
merge.data$bidOGP[is.na(merge.data$bidOGP)] <- 0 #replace the NA by 0
merge.data$choiceOGP[is.na(merge.data$choiceOGP)] <- 0

rm(a1, a2)



########################################################
##information of participants 
mean(bio.data$age)
sd(bio.data$age)

sum(bio.data$gender)




########################################################
##Response time
a1 <- data.frame(participant = biddata$participant, item = biddata$item,  RT = biddata$RT, task = 1)
a2 <- data.frame(participant = choicedata$participant, item = choicedata$item, RT = choicedata$RT, task = 0)
a3 <- rbind(a1, a2)
a3$RT <- a3$RT / 1000
a3$task <- factor(a3$task)

a3$participant <- factor(a3$participant)
a3$item <- factor(a3$item)
m <- lmer(RT ~ task + (1 + task | participant) + (1 | item), data = a3)
summary(m)
se <- sqrt(diag(vcov(m)))
(tab <- cbind(Est = fixef(m), LL = fixef(m) - 1.96 * se, UL = fixef(m) + 1.96 * se))
groupwiseMean(RT ~ task, data = a3, conf = 0.95, digits = 3)


rm(m, tab, se, a1, a2, a3)




########################################################
##Time preference reversals

#the choice task
a1 <- data.frame(participant = merge.data$participant, item = merge.data$item, chooseSS = merge.data$chooseSS, count = 1)
a2 <- summaryBy(count ~ participant + chooseSS, data = a1, FUN = sum)
participantitem <- unique(a2$participant)
choice.participant <- data.frame(participant = participantitem)
for(i in 1: length(participantitem)){
  a <- subset(a2, a2$participant == participantitem[i])
  SS <- ifelse(length(a$count.sum[a$chooseSS == 1]) > 0, a$count.sum[a$chooseSS == 1], 0)
  LL <- ifelse(length(a$count.sum[a$chooseSS == 0]) > 0, a$count.sum[a$chooseSS == 0], 0) 
  Equal <- ifelse(length(a$count.sum[a$chooseSS == 2]) > 0, a$count.sum[a$chooseSS == 2], 0)
  total <- sum(a$count.sum)
  choice.participant$pSS[i] <- SS / total
  choice.participant$pLL[i] <- LL / total
  choice.participant$pEqual[i] <- Equal / total
}

choice.participant$n <- 1
# chose the SS options both times
groupwiseMean(pSS ~ n, data = choice.participant, conf = 0.95, digits = 3)

#chose each option once
groupwiseMean(pEqual ~ n, data = choice.participant, conf = 0.95, digits = 3)

#chose the LL options both times
groupwiseMean(pLL ~ n, data = choice.participant, conf = 0.95, digits = 3)

t.test(choice.participant$pSS, choice.participant$pLL, paired = T)
cohen.d(choice.participant$pSS, choice.participant$pLL, paired = T)

rm(a, a1, a2, i, LL, SS, participantitem, total, Equal)


#the bidding task
a1 <- data.frame(participant = merge.data$participant, item = merge.data$item, SSBigerValuation = merge.data$SSBigerValuation, count = 1)
a2 <- summaryBy(count ~ participant + SSBigerValuation, data = a1, FUN = sum)
participantitem <- unique(a2$participant)
bid.participant <- data.frame(participant = participantitem)
for(i in 1: length(participantitem)){
  a <- subset(a2, a2$participant == participantitem[i])
  SS <- ifelse(length(a$count.sum[a$SSBigerValuation == 1]) > 0, a$count.sum[a$SSBigerValuation == 1], 0)
  LL <- ifelse(length(a$count.sum[a$SSBigerValuation == 0]) > 0, a$count.sum[a$SSBigerValuation == 0], 0) 
  Equal <- ifelse(length(a$count.sum[a$SSBigerValuation == 2]) > 0, a$count.sum[a$SSBigerValuation == 2], 0)
  total <- sum(a$count.sum)
  bid.participant$pSS[i] <- SS / total
  bid.participant$pLL[i] <- LL / total
  bid.participant$pEqual[i] <- Equal / total
}

bid.participant$n <- 1
# bid higher on the SS option
groupwiseMean(pSS ~ n, data = bid.participant, conf = 0.95, digits = 3)

#bid equal amounts for both options
groupwiseMean(pEqual ~ n, data = bid.participant, conf = 0.95, digits = 3)

#bid higher on the LL options
groupwiseMean(pLL ~ n, data = bid.participant, conf = 0.95, digits = 3)


t.test(bid.participant$pSS, bid.participant$pLL, paired = T)
cohen.d(bid.participant$pSS, bid.participant$pLL, paired = T)

rm(a, a1, a2, i, LL, SS, participantitem, total, Equal)


#compare the two task

# Participants preferred the SS options significantly more often when choosing than when bidding
t.test(choice.participant$pSS,bid.participant$pSS, paired = T)
cohen.d(choice.participant$pSS, bid.participant$pSS, paired = T)

# Participants preferred the LL options significantly more often when bidding than when choosing
t.test(bid.participant$pLL, choice.participant$pLL, paired = T)
cohen.d(bid.participant$pLL, choice.participant$pLL, paired = T)

rm(choice.participant, bid.participant)



########################################################
##model fitting
a1 <- data.frame(participant = choicedata$participant, item = choicedata$item, SSleft = choicedata$SSleft, adelay = choicedata$adelay, 
                 aoutcome = choicedata$aoutcome,bdelay = choicedata$bdelay, boutcome = choicedata$boutcome, choice = choicedata$choice,
                 chooseA = choicedata$chooseA) # choice task
a2 <- data.frame(participant = biddata$participant, item = biddata$item, option = biddata$option, delay = biddata$delay, outcome = biddata$outcome, 
                 valuation = biddata$valuation) # bidding task

# the discounting rate in the bidding task
a2$kt <- a2$outcome / a2$valuation - 1
a2 <- subset(a2, a2$kt != Inf)
participantitem <- unique(a2$participant)
bidk <- data.frame(participant = participantitem, bidk = 1)
for (i in 1: length(participantitem)){
  participanta2 <- subset(a2, a2$participant == participantitem[i])
  m <- lm(kt ~ delay, data =  participanta2)
  bidk$bidk[i] <- m$coefficients[2]
}
rm(participantitem, participanta2, i, m, a2)

#discount rate in the choice task
HyperbolicLikelihood <- function(k){  #the likelihood function
  likelihood <- 0
  for (i in 1: nrow(participanta1))
  {
    VA <- participanta1$aoutcome[i] / (1 + k * participanta1$adelay[i])
    VB <- participanta1$boutcome[i] / (1 + k * participanta1$bdelay[i])
    z <- VA - VB
    p <- 1 / (1 + exp(-z))
    
    if (participanta1$chooseA[i] == 1)
    {
      likelihood <- likelihood + p
    }
    else
    {
      likelihood <- likelihood + (1 - p)
    }
  }
  return(likelihood)
}

participantitem <- unique(a1$participant)
choiceLL <- data.frame(participant = participantitem, choicek = 0, LL = 0)

for (i in 1: length(participantitem)){
  participanta1 <- subset(a1, a1$participant == participantitem[i])
  x.inv <- try(out <- optimize(function(k) {HyperbolicLikelihood(k)}, interval = c(-100, 10000), maximum = T), silent = TRUE) ##忽略报错
  if ('try-error' %in% class(x.inv)) next
  choiceLL$choicek[i] <- out$maximum
  choiceLL$LL[i] <- out$objective
}

model.fit <- merge(bidk, choiceLL, by = c('participant'), all = T)
model.fit <- subset(model.fit, model.fit$choicek < 9999) #delete the four participants that are not convergent
rm(a1, bidk, out, participanta1, x.inv, i, participantitem, HyperbolicLikelihood, choiceLL)

newdata <- merge(newdata, model.fit, by = c('participant'), all = T) #merged data

model.fit$n <- 1
# k in the bidding task
groupwiseMean(bidk ~ n, data = model.fit, conf = 0.95, digits = 3)

# k in the choice task
groupwiseMean(choicek ~ n, data = model.fit, conf = 0.95, digits = 3)

t.test(model.fit$bidk, model.fit$choicek, paired = T)
cohen.d(model.fit$bidk, model.fit$choicek, paired = T)






########################################################
###Eye-tracking results

#data preparation
a1 <- data.frame(participant = biddata$participant, item = biddata$item, type = biddata$option, task = 1, 
                 MeanFixationDuration = biddata$MeanFixationDuration, OutcomeDwell = biddata$OutcomeDwell, DelayDwell = biddata$DelayDwell)
a1$OGP <- a1$OutcomeDwell / (a1$OutcomeDwell + a1$DelayDwell)
a1 <- within(a1, rm(OutcomeDwell, DelayDwell))

a2 <- data.frame(participant = choicedata$participant, item = choicedata$item,  type = choicedata$SSleft, task = 0, 
                 MeanFixationDuration = choicedata$MeanFixationDuration, 
                 OutcomeDwell = choicedata$OutcomeDwell, DelayDwell = choicedata$DelayDwell)
a2$OGP <- a2$OutcomeDwell / (a2$OutcomeDwell + a2$DelayDwell)
a2 <- within(a2, rm(OutcomeDwell, DelayDwell))

a3 <- rbind(a1, a2)
a3 <- na.omit(a3)
a3$MeanFixationDuration <- a3$MeanFixationDuration / 1000
a3$participant <- factor(a3$participant)
a3$item <- factor(a3$item)

##OGP
m <- lmer(OGP ~ task + (1 + task  | participant) + (1 | item), data = a3)
summary(m)
se <- sqrt(diag(vcov(m)))
(tab <- cbind(Est = fixef(m), LL = fixef(m) - 1.96 * se, UL = fixef(m) + 1.96 * se))

groupwiseMean(OGP ~ task, data = a3, conf = 0.95, digits = 3)


##MFD
m <- lmer(MeanFixationDuration ~ task + (1 + task| participant) + (1 | item), data = a3)
summary(m)
se <- sqrt(diag(vcov(m)))
(tab <- cbind(Est = fixef(m), LL = fixef(m) - 1.96 * se, UL = fixef(m) + 1.96 * se))

groupwiseMean(MeanFixationDuration ~ task, data = a3, conf = 0.95, digits = 3)

rm(m, tab, se)



a4 <- summaryBy(OGP + MeanFixationDuration ~ participant + task, data = a3, FUN = mean)
a5 <- subset(a4, a4$task == 1)
a6 <- subset(a4, a4$task == 0)
a7 <- data.frame(participant = a5$participant, bidOGP = a5$OGP.mean, bidMFD = a5$MeanFixationDuration.mean,
                 choiceOGP = a6$OGP.mean, choiceMFD = a6$MeanFixationDuration.mean)
cor.test(a7$bidOGP, a7$choiceOGP)
cor.test(a7$bidMFD, a7$choiceMFD)

rm(a1, a2, a3, a4, a5, a6, a7)





########################################################
### Behavioral self-report and time preference reversals

#BIS-11 score
a1 <- data.frame(participant = model.fit$participant, bidk = model.fit$bidk, choicek = model.fit$choicek)
a2 <- data.frame(participant = bio.data$participant, BIS = bio.data$BIS)
a3 <- merge(a1, a2, by = c('participant'), all = F)
a4 <- rbind(data.frame(participant = a3$participant, k = a3$bidk, task = 'bid', BIS = a3$BIS),
            data.frame(participant = a3$participant, k = a3$choicek, task = 'choice', BIS = a3$BIS))
m <- glm(k ~ BIS + task + BIS: task, data = a4)
summary(m)
confint(m)

rm(a1, a2, a3, a4, m, model.fit)

##MS score
a1 <- merge.data
a1$PreferenceReversal <- ifelse(a1$chooseSS == 1 & a1$SSBigerValuation == 0, 1, 0)
a2 <- summaryBy(PreferenceReversal ~ participant, data = a1, FUN = mean)
a3 <- data.frame(participant = bio.data$participant, MS = bio.data$MS)
a4 <- merge(a2, a3, by = c('participant'), all = F)
m <- glm(PreferenceReversal.mean ~ MS, data = a4)
summary(m)
confint(m)

rm(a1, a2, a3, a4, m)




########################################################
##data preparation for the mediation analysis in the SPSS

# bidding task
a1 <- data.frame(participant = bid.merge$participant, item = bid.merge$item, 
                 bidMFD = (bid.merge$SSTotalDwellTime + bid.merge$LLTotalDwellTime) / (bid.merge$SSTotalFixationCount + bid.merge$LLTotalFixationCount),
                 bidOGP = (bid.merge$SSOutcomeDwell + bid.merge$LLOutcomeDwell) / (bid.merge$SSTotalDwellTime + bid.merge$LLTotalDwellTime),
                 SSBiger = bid.merge$SSBigerValuation)
a1$SSBiger[a1$SSBiger == 0] <- -1
a1$SSBiger[a1$SSBiger == 2] <- 0
a1 <- na.omit(a1)

# choice task
a2 <- data.frame(participant = choice.merge$participant, item = choice.merge$item, 
                 choiceMFD = (choice.merge$SSTotalDwellTime + choice.merge$LLTotalDwellTime) / (choice.merge$SSTotalFixationCount + choice.merge$LLTotalFixationCount),
                 choiceOGP = (choice.merge$SSOutcomeDwell + choice.merge$LLOutcomeDwell) / (choice.merge$SSTotalDwellTime + choice.merge$LLTotalDwellTime),
                 chooseSS = choice.merge$chooseSS)
a2$chooseSS[a2$chooseSS == 0] <- -1
a2$chooseSS[a2$chooseSS == 2] <- 0
a2 <- na.omit(a2)

# merge the data
a3 <- merge(a1, a2, by = c('participant', 'item'), all = F)
a3 <- na.omit(a3)

write.table(a3, 'mediation.csv', row.names = F, col.names = T, sep = ", ")
rm(a1, a2, a3)







